<?php

namespace Drupal\proc\Element;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Render\Element\FormElement;
use Drupal\Core\Url;
use Drupal\proc\Entity\Proc;
use Drupal\proc\ProcInterface;

/**
 * Provides a form element for encryption and decryption.
 *
 * @FormElement("procfield")
 */
class ProcField extends FormElement {

  /**
   * {@inheritdoc}
   */
  public function getInfo(): array {
    $class = get_class($this);
    return [
      '#input' => TRUE,
      '#process' => [
        [$class, 'processProcField'],
      ],
      '#theme_wrappers' => ['form_element'],
    ];
  }

  /**
   * Check if field is disabled.
   *
   * This is used for disabling encrypt operation.
   *
   * @param array $element
   *   The element.
   * @param array $complete_form
   *   The complete form.
   *
   * @return bool
   *   TRUE if field is disabled, FALSE otherwise.
   */
  private static function procFieldIsDisabled(&$element, &$complete_form): bool {
    if (isset($complete_form[$element['#proc-settings']['proc_field_name']]['#disabled'])) {
      return TRUE;
    }
    return FALSE;
  }

  /**
   * Get default value.
   *
   * @param array $element
   *   The element.
   *
   * @return string
   *   String with proc ID if field has default value, FALSE otherwise.
   */
  private static function procGetDefaultValue(array &$element): string|bool {
    if ($element['target_id']['#default_value']) {
      return $element['target_id']['#default_value']->get('id')->getValue()[0]['value'];
    }
    return FALSE;
  }

  /**
   * Processes a proc field form element.
   */
  public static function processProcField(&$element, FormStateInterface $form_state, &$complete_form) {
    \Drupal::moduleHandler()->alter('proc_element', $element, $form_state, $complete_form);

    // Get current path:
    $current_path = \Drupal::service('path.current')->getPath();

    $config = \Drupal::config('proc.settings');
    $token_service = \Drupal::service('token');
    $proc_field_name = $element['#proc-settings']['proc_field_name'];
    $field_input_mode = $element['#proc-settings']['field_input_mode'];
    $to_recipients_field_name = $element['#proc-settings']['to_recipients_field_name'];
    $carbon_copy_recipients_field_name = $element['#proc-settings']['carbon_copy_recipients_field_name'];
    $preset_recipients_csv = $element['#proc-settings']['preset_recipients_csv'];
    $fetcher_endpoint = $token_service->replace($element['#proc-settings']['fetcher_endpoint']);
    $fetcher_endpoint_filter = $element['#proc-settings']['fetcher_endpoint_filter'];
    $fetcher_endpoint_filter_type = $element['#proc-settings']['fetcher_endpoint_filter_type'];
    $encrypt_button_label = $element['#proc-settings']['encrypt_button_label'];
    $decrypt_button_label = $element['#proc-settings']['decrypt_button_label'];
    $cache_password = $element['#proc-settings']['cache_password'];
    $trigger_format = $element['#proc-settings']['trigger_format'];
    $submit_element_id = $element['#proc-settings']['submit_element_id'];
    $trigger_fields = $element['#proc-settings']['trigger_fields'];
    $proc_field_decryption_mode = $element['#proc-settings']['proc_field_decryption_mode'];
    // If a field is disabled, and it uses a checkbox for encryption, the
    // checkbox must be replaced by a decryption button in case there is a
    // default value.
    // This way, the content remain accessible even if the field is disabled.
    $field_disabled = self::procFieldIsDisabled($element, $complete_form);
    $default_value = self::procGetDefaultValue($element);

    if ($field_disabled && $default_value) {
      // Force Decrypt button if field is disabled and has a default value.
      $trigger_format = ProcInterface::TRIGGER_FORMAT_BUTTONS;
    }

    // JavaScript variables.
    $direct_fetcher_js_prefix = <<<JS
  let to_recipients_collection = [];
  let cc_recipients_collection = [];
  let pre_set_recipients_csv = "{$preset_recipients_csv}";
  let fetcher_endpoint = "{$fetcher_endpoint}";
  let fetcher_endpoint_filter = "{$fetcher_endpoint_filter}";
  let fetcher_endpoint_filter_type = "{$fetcher_endpoint_filter_type}";
  let recipients = [];
  let default_value = "{$default_value}";

JS;

    $fetcher_to_value = <<<JS
  to_recipients_collection = Object.values(document.querySelectorAll("input[name^='{$to_recipients_field_name}']"));
JS;
    $direct_fetcher_to = !empty($to_recipients_field_name) ? $fetcher_to_value : '';

    $fetcher_cc_value = <<<JS
  cc_recipients_collection = Object.values(document.querySelectorAll("input[name^='{$carbon_copy_recipients_field_name}']"));
JS;
    $direct_fetcher_cc = !empty($carbon_copy_recipients_field_name) ? $fetcher_cc_value : '';

    $fetcher_value = <<<JS
  let proc_path = proc_path_prefix_add + "/" + ids_csv + proc_path_suffix;
  let ajaxSettings = {
    url: proc_path,
    dialogType: "dialog",
    dialog: { width: 400, classes: { "ui-dialog": "class-{$proc_field_name}" } },
  };
  let myAjaxObject = Drupal.ajax(ajaxSettings);
  myAjaxObject.execute();
JS;

    if ($fetcher_endpoint) {
      $fetcher_value = <<<JS
  // Fetcher endpoint CSV:
  let fetcher_endpoint_csv = "";
  let endpoint_response;
  let dynamic_endpoint_filter;
  let dynamic_endpoint_filter_prefix;

  if (fetcher_endpoint_filter != "") {
    // Turn fetcher_endpoint_filter CSV into an array:
    const filter_element_identifiers = fetcher_endpoint_filter.split(",");
    let filter_element_values = [];
    const identifier_type_string = fetcher_endpoint_filter_type == '0' ? 'id' : 'name';
    filter_element_identifiers.map((fieldIdentifier, fieldIndex) => {
      // Get the value of the field:
      let field_value = document.querySelector('[' + identifier_type_string + '^="' + fieldIdentifier + '"]').value;
      filter_element_values.push(fieldIdentifier + "=" + field_value);
    });
    dynamic_endpoint_filter = filter_element_values.join("&");
    dynamic_endpoint_filter_prefix = "?";
  }
  if (fetcher_endpoint != "") {
    if (dynamic_endpoint_filter) {
      fetcher_endpoint = fetcher_endpoint + dynamic_endpoint_filter_prefix + dynamic_endpoint_filter;
    }
    console.info("Fetching recipients from endpoint: " + fetcher_endpoint);

    endpoint_response = fetch(fetcher_endpoint, {
        method: "get"
    }).then(async function(response) {
      return response.json();
    }).catch(async function(err) {
      console.info("Error.");
    });
  }
  let endpoint_recipients_csv = endpoint_response.then(async function(result) {
    if (typeof result === 'object') {
      result = Object.values(result);
    }
    return result.map(item => item.id).join();
  });
  endpoint_recipients_csv.then(function(result) {
    if (result != "") {
      var separator = "";
      if (ids_csv != "") {
        separator = ",";
      }
      ids_csv = ids_csv + separator + result;
      proc_path = proc_path_prefix_add + "/" + ids_csv + proc_path_suffix;

      ajaxSettings = {
        url: proc_path,
        dialogType: "dialog",
        dialog: { width: 400, classes: { "ui-dialog": "class-{$proc_field_name}" } },
      };

      let myAjaxObject = Drupal.ajax(ajaxSettings);
      myAjaxObject.execute();
      return false;
    }
  });
JS;
    }

    $direct_fetcher_js = <<<JS
  jQuery(this).attr("href", "#");

  // Check if this is an autocomplete deluxe field:
  if (to_recipients_collection.length) {
    if (to_recipients_collection[0].value == "" && to_recipients_collection[1].value.includes('""""')) {
      // This is an autocomplete deluxe field. Get values from the second element.
      let autocomplete_deluxe_recipients_string = to_recipients_collection[1].value + cc_recipients_collection[1].value;
      const regex = /\((\d+)\)/g;
      const matches = autocomplete_deluxe_recipients_string.match(regex);
      recipients = matches.map((match) => parseInt(match.match(/\d+/)));

    }
    else {
      // Suppose this is a regular autocomplete field:
      let recipients_length = to_recipients_collection.length + cc_recipients_collection.length;
      if (recipients_length > 0) {
        let recipients_collection = to_recipients_collection.concat(cc_recipients_collection);
        recipients_collection.forEach(function (recipient, index) {
          if (index < recipients_length) {
            if (recipient.value.match(/\(\d+\)/)) {
              let id_parenthesis = recipient.value.match(/\(\d+\)/)[0];
              recipients.push(id_parenthesis.substring(1, id_parenthesis.length - 1));
            }
          }
        });
      }
    }
  }

  let proc_path_prefix_add = window.location.origin + drupalSettings.path.baseUrl + "proc/add";
  let ids_csv = recipients.join();

  // Concatenate predefined recipients:
  if (pre_set_recipients_csv != "") {
    ids_csv = ids_csv + "," + pre_set_recipients_csv;
  }

  let proc_field_name = "{$proc_field_name}";
  let parent_id = document.querySelector("input[name^='" + proc_field_name + "']").id;

  let field_input_mode = {$field_input_mode};

  let dashed_proc_field_name = proc_field_name.replace(/_/g, "-");

  let classes = this.parentNode.classList;
  // Get the class name that includes the field name:
  let proc_field_name_class_delta;
  classes.forEach(function (class_name, index) {
    if (!proc_field_name_class_delta && class_name.includes(dashed_proc_field_name)) {
      proc_field_name_class_delta = class_name;
    }
  });

  let delta = proc_field_name_class_delta.split("-").pop();

  let proc_path_suffix = "?proc_standalone_mode=FALSE&proc_parent_id=" + parent_id + "&proc_field_delta=" + delta + "&proc_field_name={$proc_field_name}&field-input-mode=" + field_input_mode + "&field-default-proc-id=" + default_value + "&host_page_url={$current_path}";

  {$fetcher_value}
  return false;
JS;

    $direct_fetcher_js = $direct_fetcher_js_prefix . $direct_fetcher_to . $direct_fetcher_cc . $direct_fetcher_js;

    // Create the URL for the Encrypt button.
    $url = Url::fromUserInput('#', [
      'attributes' => [
        'onclick' => $direct_fetcher_js,
        'id' => 'encrypt-' . $proc_field_name,
      ],
    ]);

    $link = [];

    if (!$field_disabled) {
      // Create the Encrypt button link.
      $encrypt_button_class = [$config->get('proc-encrypt-button-class')];
      if ((int) $trigger_format === ProcInterface::TRIGGER_FORMAT_CHECKBOX) {
        $encrypt_button_class = ['hidden'];
      }
      $link = [
        '#title' => ($encrypt_button_label) ? t($encrypt_button_label) : t($config->get('proc-encrypt-button-label')),
        '#type' => 'link',
        '#url' => $url,
        '#attributes' => [
          'class' => array_merge($encrypt_button_class, ['proc-link']),
          'name' => 'encrypt-' . $proc_field_name,
          'id' => 'encrypt-' . $proc_field_name,
        ],
      ];
    }

    $checkbox_onclick = <<<JS
if (this.checked) {
  document.getElementById('encrypt-{$proc_field_name}').click();
  this.value = 1;
}
else {
  // Do not try to decrypt if there is no default value and instead bring back the data-proc-plaintext attribute:
  let fieldMachineName = this.getAttribute('name').replace(/\[.+/,'');
  const input_element = document.querySelector('[name^={$proc_field_name}]');
  if (input_element.hasAttribute('data-proc-plaintext')) {
    const plaintext = input_element.getAttribute('data-proc-plaintext');
    // Remove the input clone:
    input_element.parentNode.removeChild(input_element.nextElementSibling);
    const input_original = document.querySelector('[name^={$proc_field_name}]');
    input_original.value = plaintext;
    input_original.style.display = 'block';
  }
  else {
    document.getElementById('decrypt-{$proc_field_name}').click();
  }
}
JS;
    // Create the checkbox for encryption.
    $checkbox = [
      '#type' => 'checkbox',
      '#title' => t('Encrypt'),
      '#default_value' => 0,
      '#id' => 'encrypt-checkbox-' . $proc_field_name,
      '#attributes' => [
        'onclick' => $checkbox_onclick,
        'value' => 0,
      ],
      // '#prefix' => '<div class="proc-checkbox">',
      // '#suffix' => '<span class="switch" style=""><span class="slider round"></span></span></div>',
    ];

    $element['target_id']['#attached']['library'][] = 'proc/proc-field';

    if ($submit_element_id) {
      $element['target_id']['#attached']['drupalSettings']['proc']['submit_element_id'] = $submit_element_id;
    }

    $decryption_link = [];

    $decryption_mode = FALSE;
    $proc_id = '';
    // If there is a default value, add also the Decrypt button:
    if (!empty($element['target_id']['#default_value']) && $element['target_id']['#default_value'] instanceof Proc) {

      $label = $element['target_id']['#default_value']->get('label')->getValue()[0]['value'];

      $decryption_mode = $proc_field_decryption_mode ?? FALSE;

      $proc_id = $element['target_id']['#default_value']->get('id')->getValue()[0]['value'];

      $proc_object_decryption = Proc::load($proc_id);

      $recipients = $proc_object_decryption->get('field_recipients_set')->getValue();
      $current_user_id = \Drupal::currentUser()->id();

      $current_user_recipient = TRUE;
      if (!in_array($current_user_id, array_column($recipients, 'target_id'))) {
        $current_user_recipient = FALSE;
      }

      $decryption_url = Url::fromUserInput(
        '/proc/' . $proc_id
      );
      $decryption_link_label = ($decrypt_button_label) ? t($decrypt_button_label) : t('Decrypt');
      $decryption_link = Link::fromTextAndUrl($decryption_link_label, $decryption_url);
      $decryption_link = $decryption_link->toRenderable();
      $decryption_link['#attributes']['id'] = 'decrypt-' . $proc_field_name;

      // If decryption has to happen in a dialog:
      if ($decryption_mode == ProcInterface::DECRYPT_MODE_DIALOG) {
        if ($current_user_recipient) {
          $dialog_decrypt_js = <<<JS
  let proc_path_prefix_add = window.location.origin + drupalSettings.path.baseUrl + "proc/{$proc_id}";

  let proc_field_name = "{$proc_field_name}";
  let parent_id = document.querySelector("input[name^='" + proc_field_name + "']").id;

  let field_input_mode = {$field_input_mode};
  let field_decryption_mode = {$decryption_mode};
  const cache_password = {$cache_password};

  let proc_path_suffix = "?proc_standalone_mode=FALSE&cache-password=" + cache_password + "&proc_parent_id=" + parent_id + "&proc_field_name={$proc_field_name}&field-input-mode=" + field_input_mode + "&field-decryption-mode=" + field_decryption_mode + "&trigger-fields={$trigger_fields}" + "&host_page_url={$current_path}";
  let proc_path = proc_path_prefix_add + "/" + proc_path_suffix;

  jQuery(this).attr("href", "#");

  let ajaxSettings = {
    url: proc_path,
    dialogType: "dialog",
    dialog: { width: 400, classes: { "ui-dialog": "class-{$proc_field_name}" } },
  };
  let myAjaxObject = Drupal.ajax(ajaxSettings);
  myAjaxObject.execute();
  return false;
JS;
        }
        else {
          $dialog_decrypt_js = <<<JS
  let proc_field_name = "{$proc_field_name}";
  let triggering_field = document.querySelector("input[name^='" + proc_field_name + "']");
  triggering_field.value = '';
  triggering_field.removeAttribute("disabled");
  return false;
JS;
        }

        $decryption_link_class = [$config->get('proc-encrypt-button-class')];
        if ((int) $trigger_format === ProcInterface::TRIGGER_FORMAT_CHECKBOX) {
          $decryption_link_class = ['hidden'];
        }
        $dialog_decrypt_url = Url::fromUserInput(
        // URL with recipients IDs will be defined
        // by the direct fetcher.
          '#',
          [
            'attributes' => [
              'onclick' => $dialog_decrypt_js,
              'class' => array_merge($decryption_link_class, [
                'proc-link',
                'proc-decrypt-link',
              ]),
              'id' => 'decrypt-' . $proc_field_name,
            ],
          ]
        );
        $decryption_link['#url'] = $dialog_decrypt_url;
      }
    }

    $description_proc_elements = [$link];
    if ((int) $trigger_format === ProcInterface::TRIGGER_FORMAT_CHECKBOX) {
      $description_proc_elements = [
        $link,
        $decryption_link,
        $checkbox,
      ];
    }

    $element['proc-ops'] = $description_proc_elements;

    if (isset($decryption_link['#type']) && $decryption_link['#type'] == 'link') {
      $checkbox['#attributes']['checked'] = 'checked';
      $element['proc-ops'] = [
        $link,
        $decryption_link,
        $checkbox,
      ];
      if ((int) $trigger_format === ProcInterface::TRIGGER_FORMAT_BUTTONS) {
        $decryption_link['#attributes']['class'] = [$config->get('proc-encrypt-button-class')];
        $element['proc-ops'] = [$link, $decryption_link];
      }
    }

    // If this is an encrypted text field:
    if ($field_input_mode == ProcInterface::INPUT_MODE_TEXTFIELD) {
      $element['target_id']['#type'] = 'textfield';

      $proc_object = FALSE;
      if ($proc_id) {
        $proc_object = Proc::load($proc_id);
        $element['target_id']['#disabled'] = TRUE;
      }

      $element['target_id']['#value'] = $label ?? '';

      // Add hidden field for proc_id:
      $element['proc-ops'] = [$element['proc-ops']];
      $element['target_id']['#default_value'] = $proc_object ?? '';
    }

    // If this is an encrypted Date and time field:
    if ($field_input_mode == ProcInterface::INPUT_MODE_DATE) {
      $element['target_id']['#type'] = 'date';

      $proc_object = FALSE;
      if ($proc_id) {
        $proc_object = Proc::load($proc_id);
        $element['target_id']['#disabled'] = TRUE;
      }

      $element['target_id']['#value'] = $label ?? '';

      // Add hidden field for proc_id:
      $element['proc-ops'] = [$element['proc-ops']];
      $element['target_id']['#default_value'] = $proc_object ?? '';
    }

    $element['target_id']['#attributes']['proc'] = 'true';

    return $element;
  }

  /**
   * Validates a procfield element.
   */
  public static function validateProcField(&$element, FormStateInterface $form_state, &$complete_form) {
    return $element;
  }

}
